package com.jobs.lib_v1.data.parser;

import java.net.HttpURLConnection;

import com.jobs.lib_v1.data.DataItemResult;
import com.jobs.lib_v1.data.DataJsonResult;
import com.jobs.lib_v1.fs.AssetsLoader;
import com.jobs.lib_v1.net.http.DataHttpConnection;
import com.jobs.lib_v1.settings.LocalStrings;

/**
 * 数据请求和解析类
 * 
 * 1.网络数据请求并解析 
 * 2.本地资源文件读取和解析
 */
public class DataLoadAndParser {
	/**
	 * 以GET方式请求XML数据并解析
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param URL 请求的URL地址
	 * @return DataItemResult 经过解析的数据结构
	 */
	public static DataItemResult loadAndParseData(String URL) {
		return loadAndParseData(URL, null);
	}

	/**
	 * 以通用方式请求数据并解析
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param URL 请求的 URL 地址
	 * @param postData POST 的字节数据；如果为 null 则视为 GET 方式请求
	 * @return DataItemResult 经过解析的数据结构
	 */
	public static DataItemResult loadAndParseData(String URL, byte[] postData) {
		DataItemResult retVal = new DataItemResult();
		DataHttpConnection conn = new DataHttpConnection();

		conn.responseIsXML = true;

		byte[] data = conn.Request(URL, postData);

		if (data == null || conn.getStatusCode() != HttpURLConnection.HTTP_OK) {
			retVal.hasError = true;
			retVal.message = LocalStrings.common_error_network_prefix + conn.getErrorMessage();
			retVal.message = retVal.message.trim();
			retVal.setErrorStack(conn.errorStack);
			return retVal;
		}

		XmlDataParser.parserData(data, retVal);

		return retVal;
	}

	/**
	 * 加载本地 XML 数据(/assets 目录下)
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param URL 相对 /assets 目录下的文件路径
	 * @return DataItemResult 经过解析的数据结构
	 */
	public static DataItemResult loadAndParseLocalData(String URL) {
		DataItemResult retVal = new DataItemResult();

		byte[] data = AssetsLoader.loadFileBytes(URL);

		if (data == null) {
			retVal.hasError = true;
			retVal.message = "Read local file [" + URL + "] error!";
			retVal.errorRecord(new Throwable());
			return retVal;
		}

		XmlDataParser.parserData(data, retVal);

		return retVal;
	}

	/**
	 * 以GET方式请求JSON数据并解析成 DataItemResult 对象
	 * 
	 * @author solomon.wen
	 * @date 2013-05-24
	 * @param URL 请求的URL地址
	 * @return DataItemResult 经过解析的数据结构
	 */
	public static DataItemResult loadJSONToResult(String URL) {
		return loadJSONToResult(URL, null);
	}

	/**
	 * 以通用方式请求数据并解析成 DataItemResult 对象
	 * 
	 * @author solomon.wen
	 * @date 2013-05-24
	 * @param URL 请求的URL地址
	 * @param postData Post的字节数据；如果为null则视为GET方式请求
	 * @return DataItemResult 经过解析的数据结构
	 */
	public static DataItemResult loadJSONToResult(String URL, byte[] postData) {
		return loadAndParseJSON(URL, postData).toDataItemResult();
	}

	/**
	 * 以GET方式请求JSON数据并解析成 DataJsonResult 对象
	 * 
	 * @author solomon.wen
	 * @date 2013-05-24
	 * @param URL 请求的URL地址
	 * @return DataJsonResult 经过解析的数据结构
	 */
	public static DataJsonResult loadAndParseJSON(String URL) {
		return loadAndParseJSON(URL, null);
	}

	/**
	 * 以通用方式请求数据并解析成 DataJsonResult 对象
	 * 
	 * @author solomon.wen
	 * @date 2013-05-24
	 * @param URL 请求的URL地址
	 * @param postData Post的字节数据；如果为null则视为GET方式请求
	 * @return DataJsonResult 经过解析的数据结构
	 */
	public static DataJsonResult loadAndParseJSON(String URL, byte[] postData) {
		DataHttpConnection conn = new DataHttpConnection();

		conn.responseIsXML = false;

		byte[] data = conn.Request(URL, postData);

		if (data == null || conn.getStatusCode() != HttpURLConnection.HTTP_OK) {
			DataJsonResult retVal = new DataJsonResult();
			retVal.setHasError(true);
			retVal.setMessage(LocalStrings.common_error_network_prefix + conn.getErrorMessage());
			retVal.setErrorStack(conn.errorStack);
			return retVal;
		}

		return JsonDataParser.parserJSON(data);
	}
}
