package com.jobs.lib_v1.list;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.SimpleAdapter;

import com.jobs.lib_v1.app.AppUtil;
import com.jobs.lib_v1.data.DataItemDetail;
import com.jobs.lib_v1.data.DataItemResult;

/**
 * DataListView 数据适配器
 * 
 * @author solomon.wen
 * @date 2012/09/11
 */
public class DataListAdapter extends SimpleAdapter implements OnItemClickListener {
	/** 事件监听器 **/
	private DataLoader mDataLoader = null;
	private DataLoadFinishListener finishListener = null;
	private OnItemClickListener itemClickListener = null;
	private OnItemClickListener headerClickListener = null;
	private OnItemClickListener footerClickListener = null;

	private Thread mDataLoadingThread = null;
	private DataListView mListView = null;
	private LayoutInflater mInflater = null;
	private Context mContext = null;

	private int currentPage = 0;
	private int pageSize = 10;
	private int maxPage = 0;
	private int mMaxDataCount = 0;

	private volatile boolean dataOnLoading = false;
	private volatile boolean dataLoadingNow = false;
	private volatile boolean dataLoadedOK = true;
	private volatile boolean hasFetchedData = false;

	private ListViewCell mErrorCell = new ListViewErrorCell();
	private ListViewCell mLoadingCell = new ListViewLoadingCell();
	private ListViewCell mEmptyCell = new ListViewEmptyCell();
	private ListViewCell mMoreCell = new ListViewMoreCell();
	private ListViewCell mDataCell = new ListViewCell();

	public boolean keepDataWhileRefresh = false;
	private boolean onRefreshNow = false;

	private final DataItemResult mListData = new DataItemResult();
	private DataItemResult netLoadData = null;

	public DataListAdapter(DataListView listView) {
		super(listView.getContext(), null, 0, null, null);

		mListView = listView;
		adapterInit();
	}

	public void initEditModeData() {
		if (null == mListView || !mListView.isInEditMode()) {
			return;
		}

		mListView.bindTextKey(ListViewCell.TextViewID, "title");

		DataItemDetail item;

		for (int i = 0; i < 20; i++) {
			item = new DataItemDetail();
			item.setStringValue("title", "DataListView item " + i);
			System.out.println("DataListView item " + i);
			mListData.addItem(item);
		}

		notifyDataSetChanged();
	}

	public DataListAdapter(Context context) {
		super(context, null, 0, null, null);

		mContext = context;
		adapterInit();
	}

	private void adapterInit() {
		if (null != mListView) {
			if (null == mContext) {
				mContext = mListView.getContext();
			}

			mListView.setOnItemClickListener(this, true);
			mListView.setAdapter(this);
		}

		if (null != mContext) {
			mInflater = LayoutInflater.from(mContext);
		}
	}

	public void setOnItemClickListener(OnItemClickListener listener) {
		itemClickListener = listener;
	}

	public void setOnHeaderClickListener(OnItemClickListener listener) {
		headerClickListener = listener;
	}

	public void setOnFooterClickListener(OnItemClickListener listener) {
		footerClickListener = listener;
	}

	@Override
	public void onItemClick(AdapterView<?> adapter, View viewGroup, int position, long id) {
		int headerCount = mListView.getHeaderViewsCount();

		if (position < headerCount) {
			if (null != headerClickListener) {
				headerClickListener.onItemClick(adapter, viewGroup, position, id);
			}
			return;
		}

		if (position > headerCount + getCount()) {
			if (null != footerClickListener) {
				footerClickListener.onItemClick(adapter, viewGroup, position, id);
			}
			return;
		}

		position -= headerCount;

		if (position < mListData.getDataCount()) {
			if (null != itemClickListener) {
				itemClickListener.onItemClick(adapter, viewGroup, position, id);
			}
		} else {
			if (dataLoadingNow) {
				return;
			} else {
				startLoadingData();
			}
		}
	}

	public boolean dataLoadNow() {
		return dataLoadingNow;
	}

	public boolean dataLoadError() {
		return !dataLoadedOK;
	}

	public boolean dataLoadDone() {
		if (dataLoadingNow) {
			return false;
		}

		if (!dataLoadedOK) {
			return false;
		}

		if (mListData.maxCount == mListData.getDataCount()) {
			return true;
		}

		return currentPage == maxPage;
	}

	public void setPageSize(int size) {
		if (size < 1) {
			pageSize = 10;
		} else {
			pageSize = size;
		}
	}

	public int getPageSize() {
		return pageSize;
	}

	public int getPageAt() {
		return currentPage;
	}

	public DataListView getListView() {
		return mListView;
	}

	public Context getContext() {
		return mListView.getContext();
	}

	public DataItemResult getListData() {
		return mListData;
	}

	public ListViewCell getEmptyCell() {
		return mEmptyCell;
	}

	public ListViewCell getErrorCell() {
		return mErrorCell;
	}

	public ListViewCell getDataCell() {
		return mDataCell;
	}

	public ListViewCell getMoreCell() {
		return mMoreCell;
	}

	public ListViewCell getLoadingCell() {
		return mLoadingCell;
	}

	public void setEmptyCell(ListViewCell cell) {
		if (null == cell) {
			return;
		}
		cell.copyBindTextKeys(mEmptyCell);
		mEmptyCell = cell;
	}

	public void setErrorCell(ListViewCell cell) {
		if (null == cell) {
			return;
		}
		cell.copyBindTextKeys(mErrorCell);
		mErrorCell = cell;
	}

	public void setDataCell(ListViewCell cell) {
		if (null == cell) {
			return;
		}
		cell.copyBindTextKeys(mDataCell);
		mDataCell = cell;
	}

	public void setMoreCell(ListViewCell cell) {
		if (null == cell) {
			return;
		}
		cell.copyBindTextKeys(mMoreCell);
		mMoreCell = cell;
	}

	public void setLoadingCell(ListViewCell cell) {
		if (null == cell) {
			return;
		}
		cell.copyBindTextKeys(mLoadingCell);
		mLoadingCell = cell;
	}

	public int getDataCount() {
		return mListData.getDataCount();
	}

	public void refreshData() {
		if (dataLoadingNow) {
			return;
		}

		onRefreshNow = true;
		dataLoadingNow = false;
		dataLoadedOK = true;

		if (!keepDataWhileRefresh) {
			mListData.clear();
			currentPage = 0;
			maxPage = 0;

			notifyDataSetChanged();
		}

		startLoadingData();
	}

	public void startLoadingData() {
		if (null == mDataLoader) {
			return;
		}

		if (dataLoadingNow) {
			return;
		}

		if (null != mDataLoadingThread && mDataLoadingThread.isAlive()) {
			return;
		}

		if (dataOnLoading) {
			return;
		}

		dataOnLoading = true;

		if (null != mListView) {
			mListView.setAllowAutoTurnPage(false);
		}

		adapterHandler.sendEmptyMessage(adapterHandler_setDataLoading);
	}

	public DataLoader getDataLoader() {
		return mDataLoader;
	}

	public void setDataLoader(DataLoader loader, boolean isRefreshStatus) {
		mDataLoader = loader;

		if (isRefreshStatus) {
			refreshData();
		} else {
			if (currentPage == 0 || currentPage < maxPage) {
				startLoadingData();
			}
		}
	}

	public void setDataLoadFinishListener(DataLoadFinishListener listener) {
		finishListener = listener;
	}

	public void replaceData(DataItemResult items) {
		if (dataLoadingNow) {
			return;
		}

		mListData.clear();
		currentPage = 0;
		maxPage = 0;

		dataLoadingNow = false;
		dataLoadedOK = true;

		appendData(items, true);
	}

	/**
	 * 若数据有删除，调用此方法会计算当前真实的页数
	 * 
	 * @author solomon.wen
	 * @date 2012-09-17
	 */
	public void calculateCurrentPage() {
		currentPage = (int) Math.floor((float) mListData.getDataCount() / pageSize);
	}

	public void appendData(DataItemResult items, boolean autoCalculateCurrentPage) {
		dataLoadingNow = false;
		hasFetchedData = true;
		dataLoadedOK = mListData.appendItems(items);

		if (null != items) {
			mListData.message = items.message;
		}

		if (autoCalculateCurrentPage) {
			currentPage = (int) Math.ceil((float) mListData.getDataCount() / pageSize);

			if (currentPage < 0) {
				currentPage = 0;
			}
		} else {
			if (dataLoadedOK) {
				currentPage++;
			}
		}

		if (dataLoadedOK) {
			maxPage = (int) Math.ceil((float) mListData.maxCount / pageSize);
		}

		notifyDataSetChanged();
	}

	protected final int adapterHandler_notifyDataSetChanged = 1;
	protected final int adapterHandler_appendData = 2;
	protected final int adapterHandler_setDataLoading = 3;

	protected Handler adapterHandler = new Handler() {
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case adapterHandler_notifyDataSetChanged:
				notifyDataSetChanged();
				break;

			case adapterHandler_appendData:
				boolean needAppendData = true;
				boolean dataHasRefreshed = onRefreshNow;

				if (onRefreshNow) {
					onRefreshNow = false;

					if (keepDataWhileRefresh) {
						if (null != netLoadData && !netLoadData.hasError) {
							currentPage = 0;
							mListData.clear();
						} else {
							if (mListData.isValidListData()) {
								needAppendData = false;
							}
						}
					} else {
						currentPage = 0;
					}
				}

				if (needAppendData) {
					if (null != netLoadData) {
						appendData(netLoadData, false);
						netLoadData = null;
					}
				} else {
					dataLoadingNow = false;
				}

				if (dataHasRefreshed && null != mListView) {
					mListView.onRefreshed();
				}

				if (null != mListView) {
					mListView.setAllowAutoTurnPage(true);
				}

				dataOnLoading = false;

				break;

			case adapterHandler_setDataLoading:
				final boolean lastDataLoadOK = dataLoadedOK;

				dataLoadedOK = true;
				dataLoadingNow = true;

				notifyDataSetChanged();
				try {
					mDataLoadingThread = new Thread(new Runnable() {
						public void run() {
							int fetchPage = currentPage + (lastDataLoadOK ? 1 : 0);

							if (onRefreshNow) {
								fetchPage = 1;
							}

							netLoadData = mDataLoader.fetchData(DataListAdapter.this, fetchPage, pageSize);

							adapterHandler.sendEmptyMessage(adapterHandler_appendData);
						}
					});

					if (null != mListView) {
						mListView.post(new Runnable() {
							public void run() {
								mDataLoadingThread.start();
							}
						});
					} else {
						mDataLoadingThread.start();
					}
				} catch (Exception e) {
					AppUtil.print(e);
				}

				break;
			}
		}
	};

	public void restoreStateFromBundle(Bundle bundle) {
		if (null == bundle) {
			return;
		}

		DataItemResult listData = (DataItemResult) bundle.getParcelable("listData");

		if (null != listData && listData instanceof DataItemResult) {
			appendData(listData, true);
			statusChangedNotify();
		}
	}

	public Bundle saveStateToBundle(Bundle bundle) {
		if (null == bundle) {
			bundle = new Bundle();
		}

		bundle.putParcelable("listData", getListData());

		return bundle;
	}

	public void statusChangedNotify() {
		adapterHandler.sendEmptyMessage(adapterHandler_notifyDataSetChanged);
	}

	@Override
	public void notifyDataSetChanged() {
		calculateItemsCount();

		if (null != mListView) {

			super.notifyDataSetChanged();

			if (mListView.getEnableAutoHeight()) {
				mListView.autoSetHeight();
			}
		}

		// 此处回滚代码至 rev.1336版
		// finishListener 事件只能在数据全部正确加载完成后才会调用
		// 此前邹玉叶在 rev.1406 和 rev.1467 版的改动，使得简历中心表单页出现Bug，希望下次改动时查看一下你的改动是否影响到其他地方
		if (null != finishListener && dataLoadDone()) {
			finishListener.onLoadFinished(DataListAdapter.this);
		}

		System.gc();
	}

	private void calculateItemsCount() {
		// 当前加载的数据数量小于1,可能是数据为空/数据加载中/数据加载出错，都要多一个单元格
		if (mListData.getDataCount() < 1) {
			mMaxDataCount = 1;
			return;
		}

		// 若当前加载的数据数量大于1,而且又等于数据总数，这表示操作成功
		if (mListData.maxCount == mListData.getDataCount()) {
			mMaxDataCount = mListData.maxCount;
			return;
		}

		// 当前加载数据数量小于数据总量，不是出错就是数据正在加载中，都要多一个单元格
		mMaxDataCount = mListData.getDataCount() + (maxPage > currentPage ? 1 : 0);
	}

	@Override
	public int getCount() {
		return mMaxDataCount;
	}

	@Override
	public DataItemDetail getItem(int position) {
		return mListData.getItem(position);
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	public View getCacheView(int resourceID, View cachedView, Object tag) {
		View tmpView = null;

		if (null != cachedView) {
			if (cachedView.getTag() == tag) {
				tmpView = cachedView;
			}
		}

		if (null == tmpView) {
			tmpView = mInflater.inflate(resourceID, null);
			tmpView.setTag(tag);
		}

		return tmpView;
	}

	@Override
	public View getView(int position, View cachedView, ViewGroup groupView) {
		// 当前加载的数据数量小于1,不是出错就是数据正在加载中，都要多一个单元格
		if (mListData.getDataCount() < 1) {
			if (dataLoadingNow || !hasFetchedData) {
				return mLoadingCell.getView(this, position, cachedView);
			} else if (!dataLoadedOK) {
				return mErrorCell.getView(this, position, cachedView);
			}

			return mEmptyCell.getView(this, position, cachedView);
		}

		// 当前位置若有数据，表示正常单元格
		if (position < mListData.getDataCount()) {
			return mDataCell.getView(this, position, cachedView);
		}

		if (dataLoadingNow) {
			return mLoadingCell.getView(this, position, cachedView);
		} else if (!dataLoadedOK) {
			return mErrorCell.getView(this, position, cachedView);
		}

		System.gc();

		return mMoreCell.getView(this, position, cachedView);
	}
}