package com.jobs.lib_v1.list;

import android.content.Context;
import android.graphics.Color;
import android.graphics.Rect;
import android.graphics.drawable.GradientDrawable;
import android.graphics.drawable.GradientDrawable.Orientation;
import android.os.Bundle;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.ListAdapter;
import android.widget.ListView;

import com.jobs.lib_v1.app.AppUtil;
import com.jobs.lib_v1.data.DataItemDetail;
import com.jobs.lib_v1.data.DataItemResult;

/**
 * DataListView 数据列表视图（可加载本地数据和网络数据）
 * 
 * @author solomon.wen
 * @date 2012/09/11
 */
public class DataListView extends ListView {
	protected DataListAdapter listAdapter = null; // ListView 的数据适配器
	protected boolean autoHeight = false; // ListView 中数据变化后是否自动改变高度
	private int mMaxItemCount = 0; // ListView 中子视图总数（当ListView滚动时，这个值会被赋上）
	private boolean allowAutoTurnPage = false; // 是否允许 ListView 自动加载下一页数据
	private boolean enableScroll = true; // 是否允许 ListView 滚动，一旦设置为true，listview上下左右都无法拖动
	private DataRefreshedListener onRefreshedListener = null; // ListView 状态发生变动时触发的监听器
	private long prevAutoTurnPageTime = 0;// 上次翻页时间
	protected int listViewScrollState; // eric.huang 列表滑动状态

	/**
	 * 设置 ListView 在设置数据后，自动调整自身高度
	 * 
	 * @author solomon.wen
	 * @date 2012/09/17
	 * @param enable
	 */
	public void setEnableAutoHeight(boolean enable) {
		autoHeight = enable;
	}

	/**
	 * 通知 adapter 数据发生变动，然后刷新 ListView
	 * 
	 * @author solomon.wen
	 * @date 2012/09/17
	 * @return ListViewCell
	 */
	public void statusChangedNotify() {
		listAdapter.statusChangedNotify();
	}

	/**
	 * 清除数据单元格中对文本控件的数据绑定
	 * 
	 * @author solomon.wen
	 * @date 2012/09/17
	 * @return ListViewCell
	 */
	public ListViewCell clearTextKeys() {
		return listAdapter.getDataCell().clearTextKeys();
	}

	/**
	 * 设置数据单元格的布局资源
	 * 
	 * @author solomon.wen
	 * @date 2012/09/17
	 * @param resID
	 * @return ListViewCell
	 */
	public ListViewCell setItemLayoutID(int resID) {
		return listAdapter.getDataCell().setLayoutID(resID);
	}

	/**
	 * 设置数据单元格中对应ID的文本自动绑定数据中对应的键值的数据
	 * 
	 * @param layoutItemElementID
	 * @param itemKey
	 * @return ListViewCell
	 */
	public ListViewCell bindTextKey(int layoutItemElementID, String itemKey) {
		return listAdapter.getDataCell().bindTextKey(layoutItemElementID, itemKey);
	}

	/**
	 * 批量设置数据单元格中对应ID的文本自动绑定数据中对应的键值的数据
	 * 
	 * @param layoutItemElementIDList
	 * @param itemKeys
	 * @return ListViewCell
	 */
	public ListViewCell bindTextKeys(int[] layoutItemElementIDList, String[] itemKeys) {
		return listAdapter.getDataCell().bindTextKeys(layoutItemElementIDList, itemKeys);
	}

	/**
	 * 获取 ListView 当前是否会自动调整高度
	 * 
	 * @author solomon.wen
	 * @date 2012/09/17
	 * @return boolean
	 */
	public boolean getEnableAutoHeight() {
		return autoHeight;
	}

	/**
	 * 获取数据为空时的单元格实例
	 * 
	 * @author solomon.wen
	 * @date 2012/09/17
	 * @return ListViewCell
	 */
	public ListViewCell getEmptyCell() {
		return listAdapter.getEmptyCell();
	}

	public void setOnItemClickListener(OnItemClickListener listener, boolean is_listview_real_listener) {
		if (is_listview_real_listener) {
			super.setOnItemClickListener(listener);
		} else {
			listAdapter.setOnItemClickListener(listener);
		}
	}

	public int getDataCount() {
		return listAdapter.getDataCount();
	}

	public DataItemDetail getItem(int index) {
		return listAdapter.getItem(index);
	}

	public void setOnHeaderClickListener(OnItemClickListener listener) {
		listAdapter.setOnHeaderClickListener(listener);
	}

	public void setOnFooterClickListener(OnItemClickListener listener) {
		listAdapter.setOnFooterClickListener(listener);
	}

	public void setOnItemClickListener(OnItemClickListener listener) {
		setOnItemClickListener(listener, false);
	}

	public DataListAdapter getAdapter() {
		return listAdapter;
	}

	public DataItemResult getListData() {
		return listAdapter.getListData();
	}

	public ListViewCell getErrorCell() {
		return listAdapter.getErrorCell();
	}

	public ListViewCell getDataCell() {
		return listAdapter.getDataCell();
	}

	public ListViewCell getMoreCell() {
		return listAdapter.getMoreCell();
	}

	public ListViewCell getLoadingCell() {
		return listAdapter.getLoadingCell();
	}

	public DataListView(Context context) {
		super(context);
		init(context);
	}

	public DataListView(Context context, AttributeSet attrs) {
		super(context, attrs);
		initAttrs(context, attrs);
		init(context);
	}

	public DataListView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		initAttrs(context, attrs);
		init(context);
	}

	protected void initAttrs(Context context, AttributeSet attrs) {
	}

	protected void init(Context context) {
		setDivider(new GradientDrawable(Orientation.RIGHT_LEFT, new int[] { Color.parseColor("#BEBEBE"), Color.parseColor("#BDBDBD"), Color.parseColor("#BEBEBE") })); // 统一使用灰色分割线
		setDividerHeight(1); // 统一分割线高度为1像素
		setFooterDividersEnabled(false); // 最后一条记录默认无分割线

		listAdapter = new DataListAdapter(this);
		initAutoTurnPageListener();

		listAdapter.initEditModeData();
	}

	public void setPageSize(int size) {
		listAdapter.setPageSize(size);
	}

	public void refreshData() {
		prevAutoTurnPageTime = 0;
		listAdapter.refreshData();
	}

	public void setDataLoader(DataLoader loader) {
		listAdapter.setDataLoader(loader, false);
	}

	public DataLoader getDataLoader() {
		return listAdapter.getDataLoader();
	}

	public void setDataLoadFinishListener(DataLoadFinishListener listener) {
		listAdapter.setDataLoadFinishListener(listener);
	}

	public void replaceData(DataItemResult items) {
		listAdapter.replaceData(items);
	}

	/**
	 * 若数据有删除，调用此方法会计算当前真实的页数
	 * 
	 * @author solomon.wen
	 * @date 2012-09-17
	 */
	public void calculateCurrentPage() {
		listAdapter.calculateCurrentPage();
	}

	public void appendData(DataItemResult items) {
		listAdapter.appendData(items, true);
	}

	public void setEmptyCell(ListViewCell cell) {
		listAdapter.setEmptyCell(cell);
	}

	public void setErrorCell(ListViewCell cell) {
		listAdapter.setErrorCell(cell);
	}

	public void setDataCell(ListViewCell cell) {
		listAdapter.setDataCell(cell);
	}

	public void setMoreCell(ListViewCell cell) {
		listAdapter.setMoreCell(cell);
	}

	public void setLoadingCell(ListViewCell cell) {
		listAdapter.setLoadingCell(cell);
	}

	/**
	 * 自动计算listView的高度
	 * 
	 * @author solomon.wen
	 * @date 2012-09-14
	 */
	public void autoSetHeight() {
		int totalHeight = 0;
		int dataCount = listAdapter.getCount();

		for (int i = 0; i < dataCount; i++) {
			try {
				View listItem = listAdapter.getView(i, null, this);
				listItem.measure(MeasureSpec.makeMeasureSpec(0, MeasureSpec.UNSPECIFIED), MeasureSpec.makeMeasureSpec(0, MeasureSpec.UNSPECIFIED));
				int height = listItem.getMeasuredHeight();
				totalHeight += height;
			} catch (Exception e) {
				AppUtil.print(e);
			}
		}

		ViewGroup.LayoutParams params = this.getLayoutParams();
		params.height = totalHeight + (this.getDividerHeight() * dataCount);
		this.setLayoutParams(params);
	}

	protected void onFocusChanged(boolean gainFocus, int direction, Rect previouslyFocusedRect) {
		super.onFocusChanged(gainFocus, direction, previouslyFocusedRect);

		if (gainFocus && previouslyFocusedRect != null) {
			final ListAdapter listAdapter = getAdapter();
			final int adapterCount = listAdapter.getCount();

			switch (direction) {
			case FOCUS_DOWN:
				for (int i = 0; i < adapterCount; i++) {
					if (listAdapter.isEnabled(i)) {
						setSelection(i);
						break;
					}
				}
				break;

			case FOCUS_UP:
				for (int i = adapterCount - 1; i >= 0; i--) {
					if (listAdapter.isEnabled(i)) {
						setSelection(i);
						break;
					}
				}
				break;

			default:
				break;
			}
		}
	}

	public void setAllowAutoTurnPage(boolean allow) {
		allowAutoTurnPage = allow;
	}

	private void tryAutoTurnPage() {
		if (!allowAutoTurnPage) {
			return;
		}

		// 控制自动翻页的频率，2000毫秒之内只翻一次
		long curTime = System.currentTimeMillis();
		if (curTime - prevAutoTurnPageTime < 2000) {
			return;
		}

		prevAutoTurnPageTime = curTime;

		if (listAdapter.dataLoadError()) {
			return;
		}

		if (listAdapter.dataLoadDone()) {
			return;
		}

		listAdapter.startLoadingData();
	}

	public void restoreStateFromBundle(Bundle bundle) {
		if (null == bundle) {
			return;
		}

		listAdapter.restoreStateFromBundle(bundle);

		int selectedItemPosition = bundle.getInt("selectedItemPosition");
		if (selectedItemPosition > 0) {
			setSelection(selectedItemPosition);
		}
	}

	public Bundle saveStateToBundle(Bundle bundle) {
		if (null == bundle) {
			bundle = new Bundle();
		}

		listAdapter.saveStateToBundle(bundle);

		bundle.putInt("selectedItemPosition", getSelectedItemPosition());

		return bundle;
	}

	/**
	 * 为listView绑上滚动事件，以便进行翻页控制
	 */
	void initAutoTurnPageListener() {
		this.setOnScrollListener(new OnScrollListener() {
			@Override
			public void onScrollStateChanged(AbsListView listView, int state) {
				listViewScrollState = state;

				if (mMaxItemCount >= getListData().getDataCount()) {
					tryAutoTurnPage();
				}
			}

			@Override
			public void onScroll(AbsListView listView, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
				doScrollAction(listView, listViewScrollState);

				if (firstVisibleItem + visibleItemCount >= totalItemCount) {
					tryAutoTurnPage();
				} else {
					mMaxItemCount = firstVisibleItem + visibleItemCount;
				}
			}
		});
	}

	/**
	 * 在列表滚动时做隐藏用户栏作用，主要用户下拉粉丝团隐藏用户信息栏目
	 * 
	 * @author eric.huang
	 * @date 2012-11-9
	 */
	public void doScrollAction(AbsListView listView, int scrollState) {
	}

	public boolean getScrollEnable() {
		return enableScroll;
	}

	/**
	 * 屏蔽上下和左右滑动事件
	 * 
	 * @param enable
	 */
	public void setScrollEnable(boolean enable) {
		enableScroll = enable;

		if (enableScroll) {
			this.setOnTouchListener(null);
		} else {
			this.setOnTouchListener(new OnTouchListener() {
				@Override
				public boolean onTouch(View v, MotionEvent event) {
					if (event.getAction() == MotionEvent.ACTION_MOVE) {
						return true;
					}

					return false;
				}
			});
		}
	}

	public void setOnRefreshedListener(DataRefreshedListener listener) {
		onRefreshedListener = listener;
	}

	public void onRefreshed() {
		if (null != onRefreshedListener) {
			onRefreshedListener.onRefreshed(this);
		}
	}

	/**
	 * 查看 Android 源码，觉得 layoutChildren 里面扔出的异常没有道理，就把它捕获了，避免浪费时间纠结 To hell with it, crash!
	 */
	@Override
	protected void layoutChildren() {
		// Added by solomon.wen / 2012-10-15
		try {
			super.layoutChildren();
		} catch (Exception e) {
			AppUtil.print(e);
		}
	}
}
