package com.jobs.lib_v1.net.http;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.zip.GZIPInputStream;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.StatusLine;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.params.ClientPNames;
import org.apache.http.client.params.CookiePolicy;
import org.apache.http.conn.params.ConnManagerParams;
import org.apache.http.conn.params.ConnRoutePNames;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;

import android.net.Uri;
import android.text.TextUtils;

import com.jobs.lib_v1.app.AppException;
import com.jobs.lib_v1.app.AppUtil;
import com.jobs.lib_v1.device.DeviceUtil;
import com.jobs.lib_v1.fs.AssetsLoader;
import com.jobs.lib_v1.misc.JavaReflectClass;
import com.jobs.lib_v1.net.NetworkManager;
import com.jobs.lib_v1.settings.LocalStrings;

/**
 * 通用网络请求类
 * 
 * 1.负责发起网络请求并返回二进制的字节数据
 * 2.如果请求的网址为相对路径，则会转换为绝对路径，并附加通用参数
 */
public class DataHttpConnection {
	private static JavaReflectClass mAppSettings = new JavaReflectClass("com.jobs.settings.AppSettings");

	/** 域名 **/
	public static String domain = mAppSettings.getStaticString("REQUEST_DOMAIN", "api.51job.com");

	/** 路径前缀 **/
	public static String urlAddPrefix = mAppSettings.getStaticString("REQUEST_URL_PREFIX", "/api/");

	/** 客户端名称设定 **/
	public static String userAgent = mAppSettings.getStaticString("REQUEST_USER_AGENT", "51job-android-client");

	/** 超时控制 **/
	public final static int connectTimeOut = mAppSettings.getStaticInt("REQUEST_CONN_TIMEOUT_MS", 20 * 1000);
	public final static int sockReadTimeOut = mAppSettings.getStaticInt("REQUEST_READ_TIMEOUT_MS", 50 * 1000);

	/** 请求耗时 **/
	private long startTime = 0;
	private long endTime = 0;

	/** 状态码和出错代码 **/
	private int responseCode = 0;
	private String errorMessage = "";
	public String errorStack = "";

	/** 返回数据是否为XML数据 **/
	public boolean responseIsXML = false;

	/** HTTP请求头 **/
	private Header[] headers = null;

	/**
	 * 构造函数，暂无特殊操作
	 */
	public DataHttpConnection() {
	}

	/**
	 * 获取数据加载耗费的时间
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @return long 获取返回时间
	 */
	public long getCostTime() {
		return this.endTime - this.startTime;
	}

	/**
	 * 获取HTTP状态码
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @return int HTTP状态码
	 */
	public int getStatusCode() {
		return this.responseCode;
	}

	/**
	 * 获取出错信息
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @return String 返回出错信息
	 */
	public String getErrorMessage() {
		return this.errorMessage;
	}

	/**
	 * 获取HTTP请求头
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @return Header[] 获取HTTP请求的Header信息
	 */
	public Header[] getAllHeaders() {
		return this.headers;
	}

	/**
	 * 获取指定名称的HTTP头的值
	 * 
	 * @author solomon.wen
	 * @date 2011-12-8
	 * @param headerName
	 * @return String
	 */
	public String getHeader(String headerName) {
		if (null == headers || TextUtils.isEmpty(headerName)) {
			return null;
		}

		for (Header header : headers) {
			if (header.getName().equals(headerName)) {
				return header.getValue();
			}
		}

		return null;
	}

	/**
	 * 构造完整的URL地址，如果是完整的URL地址，则不附加通用参数
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param URL 相对路径
	 * @return Uri 完整路径的Uri结构
	 */
	public static Uri buildFullURI(String URL) {
		String retURL = URL;

		/** 如果是相对路径，则要转为绝对路径 **/
		if (!retURL.startsWith("http://") && !retURL.startsWith("https://")) {
			/** 如果不是以斜杠打头，则添加路径前缀 **/
			if (!retURL.startsWith("/")) {
				retURL = urlAddPrefix + retURL;
			}

			/** 添加通用参数 **/
			if (!retURL.contains("?")) {
				retURL += "?";
			} else if (!retURL.endsWith("&")) {
				retURL += "&";
			}

			retURL += "partner=" + AssetsLoader.loadFileString("client.sign").trim();
			retURL += "&uuid=" + DeviceUtil.getUUID();
			retURL += "&version=" + AppUtil.appVersionCode();

			String guid = DeviceUtil.getAppGuid();
			if (null != guid && guid.length() == 32) {
				retURL += "&guid=" + URLEncoder.encode(guid);
			}

			/** 添加域名前缀 **/
			retURL = "http://" + domain + retURL;
		}

		return Uri.parse(retURL);
	}

	/**
	 * 从URI获取相对的路径
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param fullPath 传入的URI地址结构
	 * @return String 返回的相对路径URI
	 */
	public String relatedPathFromURI(Uri fullPath) {
		String URL = fullPath.toString();

		if (URL.length() < 10) {
			return "";
		}

		int startPos = URL.indexOf('/', 9);

		if (startPos < 9) {
			return "";
		}

		return URL.substring(startPos);
	}

	/**
	 * 发起 GET 方式的HTTP请求
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param URL HTTP请求的URL
	 * @return byte[] 返回请求后的字节数据
	 */
	public byte[] Request(String URL) {
		return this.Request(URL, null, null);
	}

	/**
	 * 发起通用的HTTP请求
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param URL HTTP请求的URL
	 * @param PostData POST提交的数据，如果为null则不提交数据，并视为GET请求
	 * @return byte[] 返回请求后的字节数据
	 */
	public byte[] Request(String URL, byte[] PostData) {
		return Request(URL, PostData, null);
	}

	/**
	 * 发起通用的HTTP请求
	 * 
	 * @author solomon.wen
	 * @date 2013-02-28
	 * @param URL HTTP请求的URL
	 * @param PostData POST提交的数据，如果为null则不提交数据，并视为GET请求
	 * @param UserAgent 用户代理名
	 * @return byte[] 返回请求后的字节数据
	 */
	public byte[] Request(String URL, byte[] PostData, String specailUserAgent) {
		byte[] retBytes = null;

		this.startTime = this.endTime = System.currentTimeMillis();
		this.responseCode = 0;
		this.errorMessage = "";

		// 网址不能为空
		if (TextUtils.isEmpty(URL)) {
			this.errorMessage = LocalStrings.common_error_network_url_invalid;
			this.errorStack = AppException.getExceptionStackInfo(new Throwable());
			return null;
		}

		// 必须有可用网络
		if (!NetworkManager.networkIsConnected()) {
			this.errorMessage = LocalStrings.common_error_no_available_network;
			this.errorStack = AppException.getExceptionStackInfo(new Throwable());
			return null;
		}

		// 构造完整的请求路径
		Uri fullUri = buildFullURI(URL);

		// 调试时打印请求的路径
		if (AppUtil.allowDebug()) {
			AppUtil.error(this, fullUri.toString());
		}

		HttpClient client = null;
		try {
			// 创建HTTP请求
			client = buildHttpClient();
			HttpUriRequest request = null;

			if (null == PostData) { // 发起 GET 请求
				HttpGet get = new HttpGet(fullUri.toString());
				request = get;
			} else { // 发起 POST 请求
				HttpPost post = new HttpPost(fullUri.toString());
				post.setHeader("Content-Type", "application/x-www-form-urlencoded");
				post.setEntity(new ByteArrayEntity(PostData));
				request = post;
			}

			// 指定 User Agent
			request.setHeader("User-Agent", TextUtils.isEmpty(specailUserAgent) ? userAgent : specailUserAgent);

			// 接受 gzip 格式编码
			request.setHeader("Accept-Encoding", "gzip");

			// 等待服务器响应
			HttpResponse response = client.execute(request);

			// 获取返回的HTTP状态码
			StatusLine status = response.getStatusLine();
			this.responseCode = status.getStatusCode();

			// 获取返回的HTTP头
			this.headers = response.getAllHeaders();

			// 获取返回的字节数据
			try {
				HttpEntity entity = response.getEntity();
				InputStream inputStream = entity.getContent();
				ByteArrayOutputStream content = new ByteArrayOutputStream();

				// 获取返回内容的编码类型
				Header contentEncoding = response.getFirstHeader("Content-Encoding");

				// GZIP 数据流
				if (contentEncoding != null && contentEncoding.getValue().toLowerCase().indexOf("gzip") > -1) {
					inputStream = new GZIPInputStream(inputStream);
				}

				// 读取数据流
				int readBytes = 0;
				byte[] sBuffer = new byte[512];
				while ((readBytes = inputStream.read(sBuffer)) != -1) {
					content.write(sBuffer, 0, readBytes);
				}

				retBytes = content.toByteArray();
			} catch (IOException e) {
				AppUtil.print(e);
				this.errorMessage = AppException.getErrorString(e, LocalStrings.common_error_network_recv_data);
				this.errorStack = AppException.getExceptionStackInfo(e);
			}
		} catch (Exception e) {
			AppUtil.print(e);
			this.errorMessage = AppException.getErrorString(e, LocalStrings.common_error_network_connect_server);
			this.errorStack = AppException.getExceptionStackInfo(e);
		}

		// 关闭打开的流和连接
		try {
			if (client != null) {
				client.getConnectionManager().shutdown();
				client = null;
			}

			System.gc();
		} catch (Exception e) {
			AppUtil.print(e);
		}

		this.endTime = System.currentTimeMillis();

		return retBytes;
	}

	/**
	 * 初始化 HTTP 请求配置
	 * 
	 * @author solomon.wen
	 * @date 2013/02/28
	 * @return HttpClient
	 */
	private HttpClient buildHttpClient() {
		HttpParams params = new BasicHttpParams();

		HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
		HttpProtocolParams.setContentCharset(params, HTTP.UTF_8);

		// Expect 100 Continue 是 HTTP 1.1 协议中的一个header属性。
		// 如果设置了 Expect 100 Continue，意味着客户端在向服务器发送数据时，可能会先向服务器发起一个请求看服务器是否愿意接受客户端将要发送的数据（一般是 HTTP Body，较大的数据块才会这样做）。
		// 因为 Expect 100 Continue 会导致客户端在向服务器发送数据是进行两次请求，这样对通信的性能方面将会受到一定的影响。 所以我们不能滥用该属性，应该通过设置 HttpProtocolParams.setUseExpectContinue(params, false); 将其关闭。
		HttpProtocolParams.setUseExpectContinue(params, false);

		// 从连接池中取连接的超时时间
		ConnManagerParams.setTimeout(params, connectTimeOut);

		// 连接到主机的超时时间
		HttpConnectionParams.setConnectionTimeout(params, connectTimeOut);

		// 网络请求超时超时时间
		HttpConnectionParams.setSoTimeout(params, sockReadTimeOut);

		// 缓冲区大小 (一般建议设成8k)
		HttpConnectionParams.setSocketBufferSize(params, 8 * 1024);

		SchemeRegistry schReg = new SchemeRegistry();
		schReg.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
		schReg.register(new Scheme("https", SSLSocketFactory.getSocketFactory(), 443));

		HttpClient client = new DefaultHttpClient(new ThreadSafeClientConnManager(params, schReg), params);

		client.getParams().setParameter(ClientPNames.COOKIE_POLICY, CookiePolicy.BEST_MATCH);
		client.getParams().setParameter(ConnRoutePNames.DEFAULT_PROXY, NetworkManager.getProxyHttpHost());

		return client;
	}
}
